#include <iostream>
#include <vector>
#include <string>
#include "histogram.h"
#include "text.h"
#include "svg.h"
#include <curl/curl.h>
#include <sstream>
#include <string>
using namespace std;


struct Input
{
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data(istream& in, bool prompt)
{
    size_t number_count;
    if (prompt) {cerr << "Enter number count :";}
    in >> number_count;
    Input In;
    In.numbers.resize(number_count);
    vector<double> numbers(number_count);
    for (size_t i = 0; i < number_count; i++)
    {in >> In.numbers[i];}
    size_t bin_count;
    if (prompt) {cerr << "Enter bin count :";}
    cin >> In.bin_count;
    return In;
}


size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<char*>(items), data_size);
    return data_size;
}


Input download(const string& address)
{
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if(curl)
    {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        curl_easy_setopt(curl, CURLOPT_VERBOSE, true);
        res = curl_easy_perform(curl);
        if (res != CURLE_OK)
            {
                curl_easy_strerror(res);
                exit(1);
            }
            curl_easy_cleanup(curl);
    }
       return input_data(buffer, false);
}

int main(int argc, char* argv[])
{
    curl_global_init(CURL_GLOBAL_ALL);
    Input in;
    if (argc > 1) {in = download(argv[1]);}
    else {in = input_data(cin, true);}
    const std::vector<double> bins = make_histogram(in.numbers, in.bin_count);
    show_histogram_svg(bins);
    return 0;
}
